@@ -1,11 +1,12 @@
 use crate::{
-    config::Config,
-    network::{self, Recipient},
-    runway::{self, Request, Response, RunwayIO, RunwayNotificationIn, RunwayNotificationOut},
-    signed::Signature,
+    network,
+    runway::{
+        self, NewestUnitResponse, Request, Response, RunwayIO, RunwayNotificationIn,
+        RunwayNotificationOut,
+    },
     units::{UncheckedSignedUnit, UnitCoord},
-    Data, DataProvider, FinalizationHandler, Hasher, MultiKeychain, Network, NodeCount, NodeIndex,
-    Receiver, Sender, Signable, SpawnHandle, UncheckedSigned,
+    Config, Data, DataProvider, FinalizationHandler, Hasher, MultiKeychain, Network, NodeCount,
+    NodeIndex, Receiver, Recipient, Sender, Signature, SpawnHandle, UncheckedSigned,
 };
 use codec::{Decode, Encode};
 use futures::{
@@ -15,37 +16,18 @@ use futures::{
 };
 use futures_timer::Delay;
 use log::{debug, error, info, trace, warn};
+use network::NetworkData;
 use rand::Rng;
 use std::{
     cmp::Ordering,
     collections::{BinaryHeap, HashSet},
     convert::TryInto,
     fmt::Debug,
+    io::{Read, Write},
+    marker::PhantomData,
     time,
 };
 
-#[derive(Debug, Encode, Decode, Clone)]
-pub(crate) struct NewestUnitResponse<H: Hasher, D: Data, S: Signature> {
-    pub(crate) requester: NodeIndex,
-    pub(crate) responder: NodeIndex,
-    pub(crate) unit: Option<UncheckedSignedUnit<H, D, S>>,
-    pub(crate) salt: u64,
-}
-
-impl<H: Hasher, D: Data, S: Signature> Signable for NewestUnitResponse<H, D, S> {
-    type Hash = Vec<u8>;
-
-    fn hash(&self) -> Self::Hash {
-        self.encode()
-    }
-}
-
-impl<H: Hasher, D: Data, S: Signature> crate::Index for NewestUnitResponse<H, D, S> {
-    fn index(&self) -> NodeIndex {
-        self.responder
-    }
-}
-
 /// A message concerning units, either about new units or some requests for them.
 #[derive(Debug, Encode, Decode, Clone)]
 pub(crate) enum UnitMessage<H: Hasher, D: Data, S: Signature> {
@@ -77,12 +59,7 @@ impl<H: Hasher, D: Data, S: Signature> UnitMessage<H, D, S> {
                 .map(|uu| uu.as_signable().data().clone())
                 .collect(),
             UnitMessage::RequestNewest(_, _) => Vec::new(),
-            UnitMessage::ResponseNewest(response) => response
-                .as_signable()
-                .unit
-                .iter()
-                .map(|uu| uu.as_signable().data().clone())
-                .collect(),
+            UnitMessage::ResponseNewest(response) => response.as_signable().included_data(),
         }
     }
 }
@@ -130,6 +107,34 @@ impl<H: Hasher, D: Data, S: Signature> PartialOrd for ScheduledTask<H, D, S> {
     }
 }
 
+#[derive(Clone)]
+pub struct LocalIO<D: Data, DP: DataProvider<D>, FH: FinalizationHandler<D>, US: Write, UL: Read> {
+    data_provider: DP,
+    finalization_handler: FH,
+    _unit_saver: US,
+    _unit_loader: UL,
+    _phantom: PhantomData<D>,
+}
+
+impl<D: Data, DP: DataProvider<D>, FH: FinalizationHandler<D>, US: Write, UL: Read>
+    LocalIO<D, DP, FH, US, UL>
+{
+    pub fn new(
+        data_provider: DP,
+        finalization_handler: FH,
+        unit_saver: US,
+        unit_loader: UL,
+    ) -> LocalIO<D, DP, FH, US, UL> {
+        LocalIO {
+            data_provider,
+            finalization_handler,
+            _unit_saver: unit_saver,
+            _unit_loader: unit_loader,
+            _phantom: PhantomData,
+        }
+    }
+}
+
 struct Member<H, D, S>
 where
     H: Hasher,
@@ -336,7 +341,7 @@ where
                     self.send_unit_message(message, Recipient::Node(recipient))
                 }
                 Response::NewestUnit(response) => {
-                    let requester = response.as_signable().requester;
+                    let requester = response.as_signable().requester();
                     let message = UnitMessage::ResponseNewest(response);
                     self.send_unit_message(message, Recipient::Node(requester))
                 }
@@ -425,14 +430,15 @@ pub async fn run_session<
     D: Data,
     DP: DataProvider<D>,
     FH: FinalizationHandler<D>,
-    N: Network<H, D, MK::Signature, MK::PartialMultisignature> + 'static,
+    US: Write,
+    UL: Read,
+    N: Network<NetworkData<H, D, MK::Signature, MK::PartialMultisignature>> + 'static,
     SH: SpawnHandle,
     MK: MultiKeychain,
 >(
     config: Config,
+    local_io: LocalIO<D, DP, FH, US, UL>,
     network: N,
-    data_provider: DP,
-    finalization_handler: FH,
     keybox: MK,
     spawn_handle: SH,
     mut exit: oneshot::Receiver<()>,
@@ -477,9 +483,9 @@ pub async fn run_session<
     };
     let runway_handle = runway::run(
         config.clone(),
+        local_io.data_provider,
+        local_io.finalization_handler,
         keybox.clone(),
-        data_provider,
-        finalization_handler,
         spawn_handle.clone(),
         runway_io,
         exit_stream,
